package com.tofly.yswzMaterials.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.yswzMaterials.custom.service.CommonUtilService;
import com.tofly.yswzMaterials.entity.dto.ReportQueryDto;
import com.tofly.yswzMaterials.service.ReportService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@RestController
@AllArgsConstructor
@RequestMapping("/report")
@Api(tags = "报表相关接口")
public class ReportController {

    private final ReportService reportService;
    private final CommonUtilService commonUtilService;

    @GetMapping("/ticketsTotalCount")
    @ApiOperation("报表数据统计")
    public ResultRespone<List<Map<String, Object>>> ticketsTotalCount(ReportQueryDto query){
        return ResultRespone.success(reportService.ticketsTotalCount(query));
    }

    @GetMapping("/getMaterialsCount")
    @ApiOperation("材料成本统计")
    public ResultRespone<Map<String, Object>> getMaterialsCount(ReportQueryDto query){
        return ResultRespone.success(reportService.getMaterialsCount(query));
    }

    @GetMapping("/queryMaterialsDetail")
    @ApiOperation("材料成本详情查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "环节类型 0 入库 1 出库 2 退库 4 增补出库 5 外卖出库",required = true,dataType = "int"),
            @ApiImplicitParam(name = "materialsName", value = "物料名称",required = true,dataType = "string"),
            @ApiImplicitParam(name = "stockName", value = "仓库名称",required = true,dataType = "string"),
    })
    public ResultRespone<Page> queryMaterialsDetail(Page page, ReportQueryDto query){
        return ResultRespone.success(reportService.queryMaterialsDetail(page, query));
    }

    @GetMapping("/getBillNo")
    @ApiOperation("编号生成")
    public ResultRespone<String> getBillNo(Integer type,Integer status){
        return ResultRespone.success(commonUtilService.getBillNo(type, status));
    }

    @GetMapping("/materialsByUserCount")
    @ApiOperation("按人员统计登记单据")
    public ResultRespone<Map<String,Object>> materialsByUserCount(ReportQueryDto query){
        return ResultRespone.success(reportService.materialsByUserCount(query));
    }

}
