package com.tofly.yswzMaterials.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.yswzMaterials.entity.*;
import com.tofly.yswzMaterials.entity.dto.ReportQueryDto;
import com.tofly.yswzMaterials.mapper.ReportMapper;
import com.tofly.yswzMaterials.service.*;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class ReportServiceImpl implements ReportService {

  private final ReportMapper reportMapper;
  private final StorageService storageService;
  private final PickingService pickingService;
  private final StocksTakeawayInfoService takeawayService;
  private final ProfitService profitService;
  private final LossService lossService;

  /**
   * 统计报表-出入库单统计
   *
   * @param query query
   * @return list
   */
  @Override
  public List<Map<String, Object>> ticketsTotalCount(ReportQueryDto query) {
    return reportMapper.ticketsTotalCount(query);
  }

  /**
   * 材料数量成本统计
   *
   * @return map
   */
  @Override
  public Map<String, Object> getMaterialsCount(ReportQueryDto query) {

    List<Map<String, Object>> materials = reportMapper.materialsCount(query);

    Map<String, Object> result = new HashMap<>(2);
    Map<String,Double> totalMap = new HashMap<>(10);
    totalMap.put("storageNumTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("storageNum")))).sum());
//    totalMap.put("storageAmountTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("storageAmount")))).sum());
    totalMap.put("pickingNumTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("pickingNum")))).sum());
//    totalMap.put("pickingAmountTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("pickingAmount")))).sum());
    totalMap.put("refundNumTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("refundNum")))).sum());
//    totalMap.put("refundAmountTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("refundAmount")))).sum());
    totalMap.put("additionNumTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("additionNum")))).sum());
//    totalMap.put("additionAmountTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("additionAmount")))).sum());
    totalMap.put("takeawayNumTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("takeawayNum")))).sum());
//    totalMap.put("takeawayAmountTotal",CollectionUtils.isEmpty(materials) ? 0D : materials.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("takeawayAmount")))).sum());
    result.put("total", totalMap);
    result.put("count", materials);

    return result;
  }

  @Override
  public Page queryMaterialsDetail(Page page,ReportQueryDto query) {
    Page result = new Page();
    Integer type = query.getType();
    Integer status = type == 0 || type == 1 ? 0 : 1;
    List<Integer> idsList = reportMapper.queryTicketsId(query,status);
    if (CollectionUtils.isNotEmpty(idsList)){
      if (type == 0 || type == 2){
        result = storageService.page(page, Wrappers.<Storage>lambdaQuery()
                .eq(type == 0, Storage::getStorageType, 0)
                .eq(type == 2, Storage::getStorageType, 1)
                .like(StringUtils.isNotBlank(query.getInput()),Storage::getBillNo, query.getInput())
                .in(Storage::getId, idsList));
      }else if (type ==1 || type == 3){
        result = pickingService.page(page, Wrappers.<Picking>lambdaQuery()
                .eq(type == 1, Picking::getPickingType, 0)
                .eq(type == 3, Picking::getPickingType, 1)
                .like(StringUtils.isNotBlank(query.getInput()),Picking::getBillNo, query.getInput())
                .in(Picking::getId, idsList));
      }else if (type == 4){
        result = takeawayService.page(page, Wrappers.<StocksTakeawayInfo>lambdaQuery()
                .like(StringUtils.isNotBlank(query.getInput()),StocksTakeawayInfo::getBillNo, query.getInput())
                .in(StocksTakeawayInfo::getId, idsList));
      }else if (type == 5){
        result = profitService.page(page, Wrappers.<Profit>lambdaQuery()
                .like(StringUtils.isNotBlank(query.getInput()),Profit::getBillNo, query.getInput())
                .in(Profit::getId, idsList));
      }else if (type == 6){
        result = lossService.page(page, Wrappers.<Loss>lambdaQuery()
                .like(StringUtils.isNotBlank(query.getInput()),Loss::getBillNo, query.getInput())
                .in(Loss::getId, idsList));
      }
    }
    return result;
  }

  @Override
  public Map<String, Object> materialsByUserCount(ReportQueryDto query) {
    Map<String,Object> result = new HashMap<>(2);
    List<Map<String, Object>> data = reportMapper.materialsByUserCount(query);
    result.put("data", data);
    Map<String,Double> totalMap = new HashMap<>(2);
    totalMap.put("sumTotalNum", CollectionUtils.isEmpty(data) ? null : data.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("totalNum")))).sum());
    totalMap.put("sumAmount", CollectionUtils.isEmpty(data) ? null : data.stream().mapToDouble(m->Double.parseDouble(String.valueOf(m.get("amount")))).sum());
    result.put("sum", totalMap);
    return result;
  }
}
